package sopaletras;

/**
 * <p>Title: Sopa de Letras</p>
 * <p>Description: Crea nuevas sopas de letras a partir de palabras que el usuario da</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Independiente</p>
 * @author Adriana Blanco Nez
 * @version 1.0
 */

public class Generador {

  Lista palabras;
  SopaLetras sopa;

  /**
   * Constructor
   */
  public Generador() {
    this.sopa = new SopaLetras();
    this.palabras = new Lista();
  }

  /**
   * Genera una sopa de letras de un tamao determinado
   * @param s Valor entero que especifica el tamao de la nueva sopa
   */
  public void generar(int s){
    int pos[] = new int[2];//fila y columna
    int direccion = -1;
    boolean orden = false;
    String palabra = palabras.siguiente();
    this.sopa.resize(s);
    while(palabra != null){
      direccion = determinarDireccion();
      orden = determinarOrden();
      pos = determinarPosicionValida(palabra,direccion);  //al azar
      if(orden == sopa.REVES){
        sopa.setPalabra(pos[0],pos[1],palabra,direccion,palabra.length()-1,orden);
      }
      else{
        sopa.setPalabra(pos[0],pos[1],palabra,direccion,0);
      }
      palabra = palabras.siguiente();
    }
    this.sopa.rellenar();
  }


  /**
   * Inserta una nueva palabra dentro de la lista
   * @param p Un valor String
   */
  public void insertar(String p){
    this.palabras.insertar(p);
  }

  /**
   * Elimina una palabra de la lista
   * @param p Un valor String
   */
  public void eliminar(String p){
    this.palabras.eliminar(p);
  }

  /**
   * Para obtener el contenido de la lista de palabras
   * @return Un vector de Strings
   */
  public String[] getLista(){
    return this.palabras.getLista();
  }

  /**
   * Genera una posicion aleatoria para colocar una palabra
   * @param s Palabra que se va a colocar
   * @param d Direccion
   * @return Un vector de 2 enteros que son las coordenadas generadas
   */
  public int[] determinarPosicionValida(String s,int d){
    int p[] = new int[2];
    boolean necesario = true;
    while(necesario){
      p[0] = (int)(Math.random() * (this.sopa.size-1));
      p[1] = (int)(Math.random() * (this.sopa.size-1));
      necesario = !sopa.posValida(p[0],p[1],s,d);
    }
    return p;
  }

  /**
   * Determina la orientacion de la palabra al azar
   * @return un valor entero entre 1 y 3
   */
  public int determinarDireccion(){
    return ((int)(1 + Math.random() * 3));
  }

  /**
   * Determina el orden de impresion de la palabra, normal o inverso
   * @return Un valor booleano False--> Normal, True--> Inverso
   */
  public boolean determinarOrden(){
    return (((int)(1 + Math.random() * 2)) == 1);
  }

  /**
   * Devuelve el contenido de la ultima sopa generada
   * @return Un String
   */
  public String getSopa(){
    return this.sopa.toString();
  }

  /**
   * Verifica si la lista de palabras tiene elementos o no
   * @return True--> Si hay palabras dentro de lista, False--> No hay palabras dentro de la lista
   */
  public boolean hayPalabras(){
    return (this.palabras.getSize() > 0);
  }

}