package sopaletras;

/**
 * <p>Title: Sopa de Letras</p>
 * <p>Description: Crea nuevas sopas de letras a partir de palabras que el usuario da</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Independiente</p>
 * @author Adriana Blanco Nez
 * @version 1.0
 */

public class Lista {


  private class Nodo{

    Nodo siguiente;
    String palabra;
    int etiqueta;

    /**
     * Constructor de un nodo que asigna valores ya existentes
     * @param e Etiqueta del nodo
     * @param p Palabra contenida por el nodo
     */
    public Nodo(int e, String p){
      this.etiqueta = e;
      this.palabra = p;
      this.siguiente = null;
    }

    /**
     * Constructor por omisin
     */
    public Nodo(){
      this.etiqueta = -1;
      this.palabra = null;
      this.siguiente = null;
    }

  }

  Nodo primero;
  Nodo ultimo;
  Nodo iterador;
  int size;

  /**
   * Constructor de una lista vacia
   */
  public Lista() {
    this.primero = null;
    this.ultimo = null;
    this.size = 0;
    this.iterador = null;
  }

  /**
   * Constructor que contiene un nodo ya existente
   * @param p Es el nodo con que se inicializa la lista
   */
  public Lista(Nodo p){
    this.primero = p;
    this.ultimo = this.primero;
    this.size = 1;
    this.iterador = null;
  }

  /**
   * Insertar una palabra al final de la lista
   * @param palabra
   */
  public void insertar(String palabra){
    Nodo temporal = new Nodo(this.size+1,palabra);
    if(this.primero == null){
      this.primero = temporal;
      this.ultimo = this.primero;
    }
    else{
      this.ultimo.siguiente = temporal;
      this.ultimo = this.ultimo.siguiente;
    }
    this.size++;
  }

  /**
   * Eliminar una palabra de la lista
   * @param p
   */
  public void eliminar(String p){
    boolean fin = false;
    if(this.primero.palabra.compareTo(p) == 0){
      primero = primero.siguiente;
      this.size--;
    }
    else{
      Nodo ultimoVisitado = primero;
      Nodo iterador = primero;
      while (!fin) {
        if (iterador.palabra.compareTo(p) == 0) {
          ultimoVisitado.siguiente = iterador.siguiente;
          if(iterador == ultimo){
            ultimo = ultimoVisitado;
          }
          this.size--;
          fin = true;
        }
        else {
          ultimoVisitado = iterador;
          iterador = iterador.siguiente;
          if (iterador == null)
            fin = true;
        }
      }
    }
  }

  /**
   * Devuelve el tamao de la lista
   * @return int
   */
  public int getSize(){
    return this.size;
  }

  /**
   * La clase lista tiene un iterador, este metodo devuelve el nodo al que apunta iterador en ese momento
   * y corre el iterador hacia el siguiente nodo, en orden secuencial
   * @return La palabra contenida dentro del nodo
   */
  public String siguiente(){
    if(this.iterador == null){
      this.iterador = primero;
    }
    else{
      if(this.iterador == this.ultimo){
        this.iterador = null;
      }
      else{
        this.iterador = this.iterador.siguiente;
      }
    }

    if(iterador != null){
      return this.iterador.palabra;
    }
    else{
      return null;
    }
  }


  /**
   * Vacia la lista, asignando valores nulos a los nodos primero y ultimo, y poniendo en cero el size
   */
  public void vaciar(){
    this.primero = null;
    this.ultimo = null;
    this.size = 0;
  }

  /**
   * Devuelve el contenido de la lista
   * @return Un vector de String
   */
  public String[] getLista(){
    String s[] = new String[this.size];
    Nodo iter = primero;
    for(int i = 0; i < this.size; i++){
      s[i] = iter.palabra;
      iter = iter.siguiente;
    }
    return s;
  }

}