package sopaletras;

/**
 * <p>Title: Sopa de Letras</p>
 * <p>Description: Crea nuevas sopas de letras a partir de palabras que el usuario da</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Independiente</p>
 * @author Adriana Blanco Nez
 * @version 1.0
 */

public class SopaLetras {


  String matriz[][];
  String matrizSol[][];
  int size;
  public static final int VERTICAL = 1;
  public static final int HORIZONTAL = 2;
  public static final int DIAGONAL = 3;
  public static final boolean REVES = true;

  /**
   * Constructor que inicializa la matriz en un tamao determinado por el usuario
   * @param s Tamao de la matriz, valor entero
   */
  public SopaLetras(int s){
    matriz = new String[s][s];
    matrizSol = new String[s][s];
    this.size = s;
    this.limpiarMatriz();
  }

  /**
   * Constructor por omisin
   */
  public SopaLetras(){
    matriz = null;
    matrizSol = null;
    this.size = 0;
  }

  /**
   * Llena la matriz con el valor por omision "?"
   */
  private void limpiarMatriz(){
    for(int f = 0; f < this.size; f++){
      for(int c = 0; c < this.size;c++){
        this.matriz[f][c] = "?";
        this.matrizSol[f][c] = "?";
      }
    }
  }

  /**
   * Para reasignar un nuevo tamao a la sopa de letras
   * @param s Tamao de la nueva matriz, valor entero
   */
  public void resize(int s){
    this.matriz = new String[s][s];
    this.matrizSol = new String[s][s];
    this.size = s;
    this.limpiarMatriz();
  }

  /**
   * Devuelve la matriz
   * @return Una matriz de String
   */
  public String[][] getMatriz(){
    return this.matriz;
  }

  /**
   * Asigna una matriz ya existente a la sopa de letras
   * @param m La matriz preasignada
   */
  public void setMatriz(String[][] m){
    this.matriz = m;
  }

  /**
   * Coloca un palabra dentro de la matriz que tiene la sopa de letras en un a posicion determinada,
   * lo hace de manera recursiva
   * @param fI Fila de la posicion
   * @param cI Columna de la posicion
   * @param palabra Valor String que se desea insertar
   * @param direccion Orientacion de la palabra: VERTICAL, HORIZONTAL o DIAGONAL
   * @param pos Siguiente posicion de la palabra que se debe insertar en la sopa
   */
  public void setPalabra(int fI, int cI, String palabra, int direccion, int pos){
    if(pos < palabra.length()){
      char c[] = new char[1];
      c[0] = palabra.charAt(pos);
      this.matriz[fI][cI] = (new String(c).toString());
      this.matrizSol[fI][cI] = (new String(c).toString());
      switch(direccion){
        case VERTICAL:
          this.setPalabra(fI+1,cI,palabra,direccion,pos+1);
          break;
        case HORIZONTAL:
          this.setPalabra(fI,cI+1,palabra,direccion,pos+1);
          break;
        case DIAGONAL:
          this.setPalabra(fI+1,cI+1,palabra,direccion,pos+1);
          break;

      }
    }

  }

  /**
   * Funciona igual que el anterior, solamente que se utiliza en caso se que se inserte la palabra
   * pero en orden inverso, como si se leyera de derecha a izquierda
   * @param fI Fila de la posicion
   * @param cI Columna de la posicion
   * @param palabra String que sera insertado
   * @param direccion Orientacion de la palabra
   * @param pos Siguiente posicion de la palabra a insertar en la matriz
   * @param reves
   */
  public void setPalabra(int fI, int cI, String palabra, int direccion,int pos, boolean reves){
    if(pos >= 0){
      char c[] = new char[1];
      c[0] = palabra.charAt(pos);
      this.matriz[fI][cI] = (new String(c).toString());
      this.matrizSol[fI][cI] = (new String(c).toString());
      switch(direccion){
        case VERTICAL:
          this.setPalabra(fI+1,cI,palabra,direccion,pos-1,reves);
          break;
        case HORIZONTAL:
          this.setPalabra(fI,cI+1,palabra,direccion,pos-1,reves);
          break;
        case DIAGONAL:
          this.setPalabra(fI+1,cI+1,palabra,direccion,pos-1,reves);
          break;

      }
    }

  }

  /**
   * Para conocer el contenido de una posicion especifica de la sopa de letras
   * @param fI Fila
   * @param cI Columna
   * @return Un String, que es el valor contenido en la celda
   */
  public String getContenido(int fI, int cI){
    if(fI >= 0 && cI >= 0 && fI < this.size && cI < this.size){
      return this.matriz[fI][cI];
    }
    else{
      return null;
    }
  }

  /**
   * Para conocer el contenido de la sopa de letras
   * @return Un String con todo el contenido de la matriz
   */
  public String toString(){
    String sopa = "";
    for(int f = 0; f < this.size; f++){
      for(int c = 0; c < this.size; c++){
        sopa += this.matriz[f][c] + " ";
      }
      sopa += "\n";
    }
    return sopa;
  }

  /**
   * Para conocer la solucion de la sopa de letras
   * @return Un String con las palabras que han sido insertadas
   */
  public String getSol(){
    String sopa = "";
    for(int f = 0; f < this.size; f++){
      for(int c = 0; c < this.size; c++){
        sopa += this.matrizSol[f][c] + " ";
      }
      sopa += "\n";
    }
    return sopa;
  }

  /**
   * Verifica que una palabra pueda ser insertada en una posicion especifica en una direccion especifica
   * @param f Fila
   * @param c Columna
   * @param s Palabra
   * @param d Direccion
   * @return Un valor booleano
   */
  public boolean posValida(int f, int c,String s, int d){
    boolean r1 = true;
    boolean r2 = true;
    switch(d){
      case SopaLetras.VERTICAL:
        r1 = (f+s.length()-1 < this.size);
        break;
      case SopaLetras.HORIZONTAL:
        r1 = (c+s.length()-1 < this.size);
        break;
      case SopaLetras.DIAGONAL:
        r1 = ((f+s.length()-1 < this.size)&&(c+s.length()-1 < this.size));
        break;
    }
    if(r1){
      r2 = this.cabe(f, c, s, d, 0);
      return (r1 && r2);
    }
    else{
      return r1;
    }
  }


  private boolean cabe(int f,int c,String s,int d,int pos){
    boolean cabe = true;
    if(pos < s.length()){
     char ch[] = new char[1];
     ch[0] = s.charAt(pos);
     if(this.matriz[f][c].compareTo("?") == 0 || (this.matriz[f][c].compareTo(new String(ch).toString()) == 0)){
       switch (d) {
         case VERTICAL:
           cabe = this.cabe(f + 1, c, s, d, pos + 1);
           break;
         case HORIZONTAL:
           cabe = this.cabe(f, c + 1, s, d, pos + 1);
           break;
         case DIAGONAL:
           cabe = this.cabe(f + 1, c + 1, s, d, pos + 1);
           break;

       }
     }
     else{
       cabe = false;
     }
   }
   else{
     cabe =  true;
   }
   return cabe;
  }

  /**
   * Rellena los espacios vacios de la sopa de letras, es decir, aquellos que contienen un "?"
   */
  public void rellenar(){
    String s[] = {"a","b","c","d","e","f","g","h","i","j","k","l","m","n","o","p","q","r","s","t","u","v","w","x","y","z"};
    int p = 0;
    for(int f = 0; f < this.size; f++){
      for(int c = 0; c < this.size; c++){
        if(this.matriz[f][c].compareTo("?") == 0){
          p = (int) (Math.random() * (s.length - 1));
          this.matriz[f][c] = s[p];
        }
      }
    }
  }

}