import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Title:        OchoReinasRecursivoYGrafico
 * Description:  Es un conjunto de clases que interactuando, llevan a cabo el problema de las Ocho Reinas Grfica y Recursivamente
 * Copyright:    Copyright (c) 2001
 * Company:      ECCI
 * @author Edgar Casasola Murillo
 * @version 1.0
 */

/**
 * 
 */
public class PruebaTablero extends JFrame {
  /**
   * TableroGrafico sobre el que se va a dibujar el Tablero 
   */ 
  TableroGrafico tableroG;
  /**
   * lleva la cuenta de las soluciones encontradas
   */
  private int contadorDeSoluciones;   
  /**
   * Constructor de la clase, inicializa las variables de la clase 
   */
  public PruebaTablero(){
    tableroG = new TableroGrafico();
    Container c = getContentPane();
    c.add(tableroG,BorderLayout.CENTER);
    setSize(400,400);
    show();
  }
  
  /**
   * Analiza si la posic&oacute;n es v&aacute;lida
   * @param f entero para la fila
   * @param c entero para la columna 
   * @return true si es v&aacute;lida o false si no
   */
  private boolean esValida(int f,int c){
    return ( (f < 8) && (f >=0 ) && (c < 8) && (c >= 0) );
    
  }
  
  /**
   * Analiza si la posic&oacute;n es v&aacute;lida
   * @param f entero para la fila
   * @param c entero para la columna 
   * @param sumaf entero para desplazarse por la fila
   * @param sumaC entero para desplazarse por la columna
   * @return true si es hay alguien o false si no
   */
  private boolean hayAlguienEnDireccion(int f, int c,int sumaF,int sumaC){
    int nF = f + sumaF;
    int nC = c + sumaC;
    boolean resultado = false;
    if (esValida(nF, nC)){
      if(tableroG.hayFicha(nF,nC))
        resultado = true;
      else
        resultado = hayAlguienEnDireccion(nF,nC,sumaF,sumaC);
      
    }
    
    return resultado;
  }
  
  /**
   * Analiza si la puede ponerse la reina en esa posici&oacute;n
   * @param f entero para la fila
   * @param c entero para la columna   
   * @return true si es puede ponerse la reian o false si no 
   */
  private boolean puedoPonerReina(int f, int c){
    int Df[] = {-1,-1,0,1,1,1,0,-1};
    int Dc[] = {0,1,1,1,0,-1,-1,-1};
    boolean resultado = (esValida(f,c)&&(!tableroG.hayFicha(f,c)) );
    for(int i= 0; ((i<8)&&(resultado));i++){
      if(hayAlguienEnDireccion(f,c,Df[i],Dc[i]))
        resultado = false;
    }
    
    return resultado;
  }
  
  /**
   * Analiza si la puede ponerse la reina en esa posici&oacute;n
   * @param numCelda entero para la columna   
   * @return true si es puede ponerse la reian o false si no 
   */
  public boolean ponerReinasR( int numCelda, int cantidadDeReinas){
    int f = numCelda/8;  //al ser un integer, este enunciado, asigna a f, el numero de fila en la cual estamos
    int c = numCelda%8;  //al ser un integer, este enunciado, asigna a f, el numero de fila en la cual estamos
    String alto = "";
    boolean haySolucion = false;
    if(numCelda<64) {
      if( puedoPonerReina(f,c) ) {
        tableroG.ponerFicha(new Reina(),f,c);// Crea una nueva reina
        tableroG.repaint();
        try{
          Thread.sleep(1); //manda a dormir al hilo que dibuja, para que se note la forma en la cual se est tratando de colocar las reinas
        }
        catch(Exception e){
          System.err.println("Error al dormir PruebaTablero");
        }
        
        if(cantidadDeReinas==1){
          haySolucion = true;
          contadorDeSoluciones++;
          contadorDeSolucionesTotal = contadorDeSoluciones;
          System.out.print("Solucion: " +contadorDeSoluciones);
          System.out.print("\n");
          /*As, cada vez que encuentra una solucin, aumenta el "contador de soluciones" en uno*/
          alto = JOptionPane.showInputDialog("SOLUCION ENCONTRADA! Para continuar ponga S");
          if ( (alto!=null) && (Character.toUpperCase(alto.charAt(0))=='S') ){
            haySolucion = false; //para encontrar todas las posibles soluciones
          }
          else{
            if(Character.toUpperCase(alto.charAt(0))=='N'){
              System.out.print("El total de Soluciones encontradas es: "+contadorDeSoluciones+"\n");
            }
          }
        }
        else{
          haySolucion = ponerReinasR(numCelda+1, cantidadDeReinas-1);
        }
        if(!haySolucion){
          Ficha reina = tableroG.sacarFicha(f,c);
          haySolucion = ponerReinasR(numCelda+1, cantidadDeReinas);
        }
      }
      else {
        haySolucion = ponerReinasR(numCelda+1,cantidadDeReinas);
      }
    }
    return haySolucion;
  }
  /**
   * Llama al m&eacute;todo recursivo de colocar reinas 
   */
  public void ponerReinas(){
    ponerReinasR(0,8);
  }
  
  /**
   * Controlador de la clase
   */
  public static void main(String s[]){
    PruebaTablero mesa = new PruebaTablero();
    
    mesa.ponerReinas();
    
    mesa.addWindowListener(
                           new WindowAdapter(){
      public void windowClosing(WindowEvent e){
        System.exit(0);
      }
    }
                           
    );
    
  }
  
}
