import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Title:        OchoReinasRecursivoYGrafico
 * Description:  Es un conjunto de clases que interactuando, llevan a cabo el problema de las Ocho Reinas Grfica y Recursivamente
 * Copyright:    Copyright (c) 2001
 * Company:      ECCI
 * @author Edgar Casasola Murillo
 * @version 1.0
 */

/*En esta clase, por medio de una serie de mtodos, se "colocan o se sacan" las fichas y se dibuja el tablero*/ 
/**
 * En el Tablero, se ponen y quitan fichas; adems se controla si tiene una ficha en X casilla.
 * Sabe cmo dibujarse.
 */
public class Tablero {
  
  /**
   * Representacin vectorial del tablero 
   */
  private char T[][];
  
  /**
   * Vector de Fichas 
   */
  private Ficha fichas[][];
  
  /**
   * Constructor de la clase.
   * Inicializa las variables de la clase
   */
  public Tablero(){
     char colores []= {'N','B'};  // N = negro B=Blanco
     int colorInicialDeLaFila = 0; // 0 Inicia con negro 1 con blanco
     int color = 0;
     T = new char [8][8];
     fichas = new Ficha [8][8];
     for(int f=0; f<8;f++){
        color = colorInicialDeLaFila;
        for(int c=0; c<8; c++){
           T[f][c]= colores[color];
           // Calcula el color de la siguiente celda
           color = (color + 1) % 2;
           fichas[f][c] = null;
        }
        // Calcula el color con que inicia la siguiente fila
        colorInicialDeLaFila = (colorInicialDeLaFila + 1) % 2;
     }
   }
  
   /**
    * dibuja el tablero
    */
   public void dibujar(){
      for(int f=0; f<8;f++){
        for(int c=0; c<8; c++){
           if(fichas[f][c]!=null)
              System.out.println(fichas[f][c].getSimbolo());
           else
              System.out.print(T[f][c]);
        }
        System.out.println();
      }

   }

   /**
    * Coloca una ficha 
    * @param ficha la Ficha a poner
    * @param f entero para la fila
    * @param c entero para la columna
    */
   public void ponerFicha(Ficha ficha,int f,int c){
      fichas[f][c] = ficha;
   }

   /**
    * Revisa si hay o no ficha en una posicion determinada
    * @return true si hay ficha, false en caso contrario
    * @param f entero para la fila
    * @param c entero para la columna
    */
   public boolean hayFicha(int f,int c){
      return (fichas[f][c]!=null);
   }
   
   /**
    * Devuelve la ficha que hay en una posici&oacute;n espec&iacute;fica
    * @param f entero para la fila
    * @param c entero para la columna
    */
   public Ficha sacarFicha(int f,int c){
     Ficha temp;
     temp = fichas[f][c];
     fichas[f][c]=null;
     return temp;
   }

   /**
    * Representa gr&aacute;ficamente el tablero
    * @param p instancia de JPanel
    * @param g instancia de Graphics
    */
   public void dibujar(JPanel p,Graphics g){
      for(int f=0; f<8;f++){
        for(int c=0; c<8; c++){
           if(T[f][c]=='N')
              g.setColor(Color.black);
           else
              g.setColor(Color.white);
           g.fillRect(c*40+20,f*40+20,40,40);

           if(fichas[f][c]!=null){
              fichas[f][c].dibujar(p,g,c*40+20,f*40+20,40,40);
           }

        }
      }


   }
   public static void main(String s[]){
      Tablero tablero = new Tablero();
      tablero.dibujar();
      System.exit(0);
   }
}


