import javax.swing.JOptionPane;
/* La clase vector se utiliza para demostrar la forma de declarar
 * inicializar y acceder a las celdas de un vector
 * Su diseo y estructura obedece solo a fines de demostracin
 * Programado por: MSc. Edgar Casasola Murillo (c)2004
 */
public class Vector {
  /** Declara una referencia a un vector de enteros como parte de la clase
   */
  private double vector[];
  /** <pre>
   * Constructor 
   * REQUIERE: Que el atributo vector haya sido inicializado
   * EFECTUA: Recorre el vector y presenta en la pantalla de consola el contenido de las celdas desde la posicin 0
   * hasta la ltima posicin
   * MODIFICA: La salida a consola.
   * El constructor iniciaciza un vector que contendr tantos enteros
   * como se indique en el parmetro cantidadDeElementos
   * @param cantidadDeElementos Determina el tamao que tendr el vector de enteros
   </PRE> */
  public Vector(int cantidadDeElementos){
    
    vector = new double [cantidadDeElementos];
  }
  
  /** <pre>
   * REQUIERE: Que el atributo vector haya sido inicializado
   * EFECTUA: Recorre el vector y presenta en la pantalla de consola el contenido de las celdas desde la posicin 0
   * hasta la ltima posicin
   * MODIFICA: La salida a consola.
   </PRE> */
  public String toString(){
    String resultado = "{";
    if(vector.length>0)
      resultado+=vector[0];
    for(int i=1;i<vector.length; i++){
      resultado+=","+vector[i];
    }
    resultado+= "}";
    return resultado;
  }
  /** <pre>
   * REQUIERE: Que el indice sea una posicin vlida dentro del vector
   * EFECTUA: Asigna a la celda "i" el valor recibido como parmetro
   * MODIFICA: La celda "i" del vector 
   * @param indice Indice de la celda donde se guardara el valor
   * @param valor Valor que se asignar a la celda "i"
   </PRE> */
  public void setValor(int indice, double valor){
    if((indice>=0)&&(indice<vector.length))
      vector[indice]=valor;
  }
  /** <pre>
   * REQUIERE: Que el indice sea una posicin vlida dentro del vector
   * EFECTUA: Retorna el valor de la celda "i" 
   * MODIFICA: No modifica valores externos
   * @return El valor vector[indice]
   </PRE> */
  public double getValor(int indice){
    if((indice>=0)&&(indice<vector.length))     
      return vector[indice];
    else {
      System.err.println("Error: posicin invlida "+indice+" en vector de tamao "+vector.length);
      return 0;
    }
  }
  /** <pre>
   * REQUIERE: Que el vector sea diferente de null
   * EFECTUA: Retorna el valor del tamao del vector
   * MODIFICA: No modifica valores externos
   * @return El tamao del vector
   </PRE> */
  public int length(){
    return vector.length;
  }
  /** <pre>
   * REQUIERE: Que el vector sea diferente de null
   * EFECTUA: Retorna la sumatoria de todos los elementos del vector 
   * MODIFICA: No modifica valores externos
   * @return La sumatoria de todos los valores del vector
   </PRE> */
  public double suma(){
    double resultado=0;
    for(int i=0;i<vector.length; i++){
      resultado+=vector[i];
    }
    return resultado;
  }
  /** <pre>
   * REQUIERE: Que el vector sea diferente de null
   * EFECTUA: Retorna el promedio de los valores del vector
   * MODIFICA: No modifica valores externos
   * @return El promedio de los valores del vector
   </PRE> */
  public double promedio(){
    double resultado=suma();
    if(vector.length>0)
      resultado/=vector.length;
    return resultado;
  }
  /** <pre>
   * REQUIERE: Que el vector sea diferente de null
   * EFECTUA: Retorna la posicin del elemento con el valor mnimo del vector
   * MODIFICA: No modifica valores externos
   * @return La posicin dentro del vector donde se ubica el valor ms bajo
   </PRE> */
  public int minimo(){
    int resultado= -1;
    if(vector.length > 0)
      resultado =0;    for(int i=1;i<vector.length; i++){
      if(vector[resultado]>vector[i])
        resultado = i;
    }
    
    return resultado;
  }
  /** <pre>
   * REQUIERE: Que el vector sea diferente de null
   * EFECTUA: Retorna la posicin del elemento con el valor mximo del vector 
   * MODIFICA: No modifica valores externos
   * @return La posicin de la celda del vector donde se ubica el valor ms alto
   </PRE> */
  public int maximo(){
    int resultado= -1;
    if(vector.length > 0)
      resultado =0;
    for(int i=1;i<vector.length; i++){
      if(vector[resultado]<vector[i])
        resultado = i;
    }
    return resultado;
  }
  /** <pre>
   * REQUIERE: Que el vector sea diferente de null y las posiciones "i" y "j" sean vlidas
   * EFECTUA: Intercambia el valor de la posicin "i" con el de la posicin "j"
   * MODIFICA: El vector, especificamente las celdas vector[i] y vector[j]
   </PRE> */  
  private void intercambiar(int i,int j){
    double temp = vector[i];
    vector[i]=vector[j];
    vector[j]=temp;
  }
  /** <pre>
   * REQUIERE: Que el vector sea diferente de null
   * EFECTUA: void
   * MODIFICA: Ordena de menor a mayor los elementos del vector utilizando el algoritmo de ordenamiento de burbuja
   </PRE> */
  public void ordenar(){
    int cantidadDeRecorridos = vector.length-1; // Hay que hacer tantos recorridos como elementos hay
    // Como hay length cantidad de elementos hay que hacer esa cantidad de recorridos menos 1
    // ya que en cada iteracin se compara una celda con "la siguiente"   
    int posicionLimite = vector.length-1; // El indice llega hasta este lmite length-1
    for(int recorrido=0; recorrido< cantidadDeRecorridos; recorrido++) {
      // Recorre desde el inicio hasta llegar a donde estn los elementos ya ordenados
      for(int pos=0; pos < posicionLimite; pos++) {
        // si encuentra uno mayor al siguiente lo intercambia
        if(vector[pos] > vector[pos+1]){
          intercambiar(pos,pos+1);
        }
      }
      posicionLimite--; // El mayor qued al final asi que la prxima llega hasta una posici antes
    }
  }
}