

/**
 * Esta clase implementa un jugador del juego de Cuatro en Lnea
 * @version 1.0
 */

public class Jugador {
  private String nombre;
  private String ColorFichasJugador;

  /**
   * Crea un Jugador en el juego.
   * @param pnombre El nombre del jugador
   * @param pcolorfichasjugador Para identificar el color de fichas que el jugador tiene asignadas
   *
   */
  public Jugador(String pnombre, String pcolorfichasjugador) {
    setNombre(pnombre);
    setColorFichasJugador(pcolorfichasjugador);
  }

  /**
   * Asigna el nombre al jugador
   * @param pnombre El nombre que se asignar al jugador
   */
  public void setNombre(String pnombre){
    nombre = pnombre;
  }

  /**
   * Obtiene el nombre del jugador
   * @return Un string con el nombre del jugador
   */
  public String getNombre(){
    return nombre;
  }

  /**
   * Asigna un color con el cual jugar el jugador
   * @param pcolorfichasjugador  El color de fichas que el jugador usar
   */
  public void setColorFichasJugador(String pcolorfichasjugador){
    ColorFichasJugador = pcolorfichasjugador;
  }

  /**
   * Obtiene el color de fichas que un jugador tiene
   * @return Un string con el color de fichas que el jugador tiene
   */
  public String getColorFichasJugador(){
    return ColorFichasJugador;
  }


  /**
   * Guarda una ficha en una posicin del tablero
   * @param columna La columna en que se desea depositar la ficha
   * @param pColorFicha El color de la ficha que se quiere
   */
  public boolean juega (Tablero ptablero, int pfila, int pcolumna){
    ptablero.putFicha(pfila, pcolumna, this.getColorFichasJugador());
    return this.Gano(ptablero, pfila, pcolumna);
  }


  /**
   * Determina si el jugador ha ganado
   * @param pcolor Color a buscar
   * @return Verdadero si encontr cuatro fichas en lnea y falso en caso contrario
   */
  public boolean Gano(Tablero ptablero, int pfila, int pcolumna) {
      if (ptablero.CuentaVertical(this.getColorFichasJugador(), pfila, pcolumna) == 4) return true;
      if (ptablero.CuentaHorizontal(this.getColorFichasJugador(), pfila, pcolumna)==4) return true;
      if (ptablero.CuentaDiagonal(this.getColorFichasJugador(), pfila, pcolumna)==4) return true;
      return false;
  }


  public boolean equals(Object obj){
    boolean resultado=false;
    Jugador otroJugador=null;
    if(obj instanceof Jugador) {
      otroJugador = (Jugador) obj;
      if(otroJugador==this){
        resultado = true;
      }
      else {
        if ((this.nombre==otroJugador.nombre)&&(this.ColorFichasJugador==otroJugador.ColorFichasJugador)){
          resultado = true;
        }
      }
    }
    return resultado;
  }


}
