

/**
 * <p>Titulo: Clase Tablero</p>
 * <p>Descripcion: Define la clase Tablero para el juego Cuatro en Lnea</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * @version 1.0
 */

public class Tablero {

  public final static int MAX_FILAS = 6;
  public final static int MAX_COLUMNAS = 7;

  final static int TOTAL_FICHAS = MAX_FILAS * MAX_COLUMNAS;
  private int celdasllenas;
  private Ficha[][] espacio;

  /**
   * Devuelve la ficha que se encuentra almacenada en la celda.
   * Si la celda est vaca devuelve null.
   * @param fila  Fila del Tablero en que se encuentra la ficha
   * @param columna Columna del Tablero en que se encuentra la ficha
   * @return Ficha El objeto de la clase Ficha
   */
  public Ficha getCelda (int fila, int columna){
    return espacio[fila][columna];
  }

  /**
   * Guarda una ficha en una posicin del tablero
   * @param fila La fila es que se va a depositar la ficha
   * @param columna La columna en que se desea depositar la ficha
   * @param pColorFicha El color de la ficha que se quiere depositar
   */
  public void putFicha (int pfila, int pcolumna, String pcolorficha){
     espacio[pfila][pcolumna] = new Ficha(pcolorficha);
     this.IncrementaCeldasLlenas();
  }

  /**
   * Cuenta Cantidad de Fichas en sentido vertical, de la posicin de la ficha recin insertada
   * @param pcolor Color de la ficha
   * @param pfila Fila en que est la ficha
   * @param pcolumna Columna en que est la ficha
   */
  public int CuentaVertical(String pcolor, int pfila, int pcolumna){
    int cuentafichas = 1;
    for ( int i = pfila + 1; i < MAX_FILAS; i++){
      if (espacio[i][pcolumna] != null) {
        if (espacio[i][pcolumna].getColor() == pcolor) {
          cuentafichas++;
        }
        else {
          i = MAX_FILAS;
        }
      }
    }
    return cuentafichas;
  }

  /**
   * Cuenta Cantidad de Fichas en sentido horizontal tanto a la izquierda como a
   * la derecha de la posicin de la ficha recin insertada
   * @param pcolor Color de la ficha
   * @param pfila Fila en que est la ficha
   * @param pcolumna Columna en que est la ficha
   */
  public int CuentaHorizontal(String pcolor, int pfila, int pcolumna){
    int cuentafichasder = this.CuentaHorizontalDerecha(pcolor, pfila, pcolumna);
    int cuentafichasizq = this.CuentaHorizontalIzquierda(pcolor, pfila, pcolumna);
    int cuentafichas = cuentafichasder + cuentafichasizq + 1;
    return cuentafichas;
  }

  /**
   * Cuenta Cantidad de Fichas en sentido horizontal hacia la derecha,
   *  de la posicin de la ficha recin insertada
   * @param pcolor Color de la ficha
   * @param pfila Fila en que est la ficha
   * @param pcolumna Columna en que est la ficha
   */
  private int CuentaHorizontalDerecha(String pcolor, int pfila, int pcolumna){
    int pcuentafichas = 0;
    for ( int j = pcolumna + 1; j < MAX_COLUMNAS; j++){
      if (espacio[pfila][j] != null) {
        if (espacio[pfila][j].getColor() == pcolor) {
          pcuentafichas++;
        }
        else {
          j = MAX_COLUMNAS;
        }
      }
    }
    return pcuentafichas;
  }

  /**
   * Cuenta Cantidad de Fichas en sentido horizontal hacia la izquierda,
   *  de la posicin de la ficha recin insertada
   * @param pcolor Color de la ficha
   * @param pfila Fila en que est la ficha
   * @param pcolumna Columna en que est la ficha
   */
  private int CuentaHorizontalIzquierda(String pcolor, int pfila, int pcolumna){
    int pcuentafichas = 0;
    for ( int j = pcolumna - 1; j > -1; j--){
      if (espacio[pfila][j] != null) {
        if (espacio[pfila][j].getColor() == pcolor) {
          pcuentafichas++;
        }
        else {
          j = -1;
        }
      }
    }
    return pcuentafichas;
  }

  /**
   * Cuenta Cantidad de Fichas en sentido diagonal tanto a la izquierda como a
   * la derecha de la posicin de la ficha recin insertada
   * @param pcolor Color de la ficha
   * @param pfila Fila en que est la ficha
   * @param pcolumna Columna en que est la ficha
   */
  public int CuentaDiagonal(String pcolor, int pfila, int pcolumna){
    int cuentafichasder = this.CuentaDiagonalDerecha(pcolor, pfila, pcolumna);
    int cuentafichasizq = this.CuentaDiagonalIzquierda(pcolor, pfila, pcolumna);
    int cuentafichas = cuentafichasder + cuentafichasizq + 1;
    return cuentafichas;
  }

  /**
   * Cuenta Cantidad de Fichas en sentido diagonal hacia la derecha,
   *  de la posicin de la ficha recin insertada
   * @param pcolor Color de la ficha
   * @param pfila Fila en que est la ficha
   * @param pcolumna Columna en que est la ficha
   */
  private int CuentaDiagonalDerecha(String pcolor, int pfila, int pcolumna){
 int pcuentafichas = 0;
 int j = pcolumna + 1;
   try {    
  if (j<MAX_COLUMNAS){
    for (int i = pfila + 1; i < MAX_FILAS; i++) {
   if (espacio[i][j] != null) {
     if (espacio[i][j].getColor() == pcolor) {
    pcuentafichas++;
     }
     else {
    i = MAX_FILAS;
     }
   }
   if (j < MAX_COLUMNAS) {
     j++;
   }
   else {
     i = MAX_FILAS;
   }
    }
    }
    }
    catch (ArrayIndexOutOfBoundsException e){
     e.printStackTrace();
    }
    return pcuentafichas;
  }

  /**
   * Cuenta Cantidad de Fichas en sentido diagonal hacia la izquierda,
   *  de la posicin de la ficha recin insertada
   * @param pcolor Color de la ficha
   * @param pfila Fila en que est la ficha
   * @param pcolumna Columna en que est la ficha
   */
  private int CuentaDiagonalIzquierda(String pcolor, int pfila, int pcolumna){
    int pcuentafichas = 0;
    int j = pcolumna -1;
    try{
        if (j>=0) {
       for (int i = pfila + 1; i < MAX_FILAS; i++) {
         if (espacio[i][j] != null) {
           if (espacio[i][j].getColor() == pcolor) {
             pcuentafichas++;
           }
           else {
             i = MAX_FILAS;
           }
         }
         if (j != 0) {
            j--;
         }
        else {
          i = MAX_FILAS;
        }
       }
     }
    }
 catch (ArrayIndexOutOfBoundsException e){
   e.printStackTrace();
  }
    return pcuentafichas;
  }


  /**
  * Determina si quedan espacios vacos en el tablero
  * @return Verdadero si todavia hay espacios libres y Falso en caso contrario
  */
  public boolean EspaciosLibres() {
    return celdasllenas < TOTAL_FICHAS;
  }

  /**
   * Identifica la fila en que se debe guardar una ficha, considerando la siguiente
   * celda vaca de una columna, de abajo hacia arriba
   * @param columna columna en la cual va a realizar la bsqueda
   * @return Nmero de fila disponible. Devuelve -1 si no encuentra espacio
   */
  public int BuscaFila (int columna){
    int fila = MAX_FILAS - 1;
    int flag = 0;
    while ((fila>=0) && (flag == 0)){
      if (espacio[fila][columna] == null) {
        flag = 1;
      }
      if (flag != 1){ fila--;}
    }
    return fila;
  }

  /**
   * Obtiene la cantidad de celdas llenas que hay en el Tablero
   * @param Tablero sobre el cual se desea conocer la cantidad de celdas llenas
   * @return Cantidad de celdas llenas
   */
  public int getCeldasLlenas (Tablero ptablero){
    return this.celdasllenas;
  }

  /**
   * Incrementa en 1 la cantidad de celdas llenas
   */
  private void IncrementaCeldasLlenas (){
    this.celdasllenas++;
  }


  /**
   * Inicializa el tablero en blanco o vaco
   */
  public Tablero() {
    espacio = new Ficha[MAX_FILAS][MAX_COLUMNAS];
    celdasllenas = 0;
  }
}

